import { NextFunction, Request, Response } from "express";
import Profile, { Role } from "../../models/ProfileSchema";

import { ValidationError } from "../utils/ValidationError";

const addUser = () => async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  if (!req.session!.user) {
    return res.status(401).json({ message: "You have to login to register!" });
  }

  // Already registered
  if (req.session?.user?.isRegistered) {
    await Profile.findOne(
      { external_id: req.session.user.id },
      (error, profile) => {
        if (error) {
          console.warn(error);
          return res.status(400);
        } else {
          res.status(200);
          res.data = { profile };
        }
      }
    );
    return next();
  }
  // Register
  try {
    const profile = new Profile();

    const fields = [
      { name: "studentCardNumber", required: true },
      { name: "roomNumber", required: true },
      { name: "picture", required: false },
    ];
    fields.forEach((field) => {
      const value = req.body[field.name];
      if (field.required && !value) {
        res.status(400);
        throw new ValidationError(400, `Field: {${field.name}} is required!`);
      }
      if (value) profile.set(field.name, value);
    });

    profile.external_id = req.session!.user!.id;
    profile.email = req.session?.user?.email;
    profile.name = req.session?.user?.name;

    profile.save((err) => {
      if (err) {
        res.status(400);
      } else {
        res.status(201);
        res.data = { profile };
        req.session!.user!.isRegistered = true;
      }
      next();
    });
  } catch (error) {
    if (error instanceof ValidationError) {
      const { code, message } = error;
      return res.status(code).send({ message });
    }
  }
};

export default addUser;
